package task

import (
	"fmt"
	"github.com/gogf/gf/v2/frame/g"
	"github.com/gogf/gf/v2/os/glog"
	"github.com/robfig/cron"
	"golang.org/x/crypto/ssh"
	"golang.org/x/net/context"
	"net"
	"nodeMonitor/internal/model"
	"nodeMonitor/internal/service"
	"nodeMonitor/internal/telegram"
	"nodeMonitor/internal/websocket"
	"nodeMonitor/library/nettools"
	"sync"
	"time"
)

func Ping(ctx context.Context) {
	glog.Debug(ctx, "start ping .....")
	var wg sync.WaitGroup
	//获取节点数据
	nodeList, err := service.Node().GetNode(ctx)
	if err != nil {
		glog.Debug(ctx, err.Error())
		return
	}
	for _, target := range nodeList {
		wg.Add(1)
		//glog.Debug(ctx, target)
		go PingTask(ctx, target.Host, &wg, target.PingType, target.Port, target.Id)
	}

	wg.Wait()
	//go PingStatus(ctx)
}

func PingStatus(ctx context.Context, nextTime time.Time) {
	CheckNodeStatus(ctx, nextTime)
}

func PingTask(ctx context.Context, t string, wg *sync.WaitGroup, pingType int, hostPort int, serverid int) {
	//var ipSlice []string
	//ipSlice = append(ipSlice, "kdvkr-02.xyz")
	//ipSlice = append(ipSlice, "kdvkr-04.xyz")
	//pingCount, err := g.Cfg().Get(ctx, "node.pingCount")
	//if err != nil {
	//	glog.Debug(ctx, err.Error())
	//	return
	//}
	pingconfig, err := service.PingConfig().Get(ctx)
	if err != nil {
		glog.Debug(ctx, err.Error())
		return
	}
	stat := model.PingSt{}
	stat.MinDelay = -1
	lossPK := 0
	addr, err := net.ResolveIPAddr("ip", t)
	if err == nil {
		for i := 0; i < pingconfig.NodeCount; i++ {
			starttime := time.Now().UnixNano()

			var rest nettools.IPingResult
			if pingType == 0 { //icmp
				icmping := nettools.NewIcmpPing(addr.String(), time.Second*4)
				rest = icmping.Ping()
			} else if pingType == 1 {
				tcpping := nettools.NewTcpPing(addr.String(), hostPort, time.Second*4)
				rest = tcpping.Ping()
			}

			if rest.Error() == nil {
				delay := rest.Result()
				glog.Debug(ctx, "[func:IcmpPing] Finish Addr:", addr, delay, "ms")
				stat.AvgDelay = stat.AvgDelay + rest.Result()
				if stat.MaxDelay < delay {
					stat.MaxDelay = delay
				}
				if stat.MinDelay == -1 || stat.MinDelay > delay {
					stat.MinDelay = delay
				}
				stat.RevcPk = stat.RevcPk + 1
			} else {
				glog.Debug(ctx, "[func:StartPing IcmpPing] ID:", i, " IP:", addr, "| err:", rest.Error())
				lossPK = lossPK + 1
			}
			stat.SendPk = stat.SendPk + 1
			stat.LossPk = int((float64(lossPK) / float64(stat.SendPk)) * 100)
			duringtime := time.Now().UnixNano() - starttime
			time.Sleep(time.Duration(3000*1000000-duringtime) * time.Nanosecond)
		}
		if stat.RevcPk > 0 {
			stat.AvgDelay = stat.AvgDelay / stat.RevcPk
		} else {
			stat.AvgDelay = 0.0
		}
		glog.Debug(ctx, "[func:IcmpPing] Finish Addr:", addr, " MaxDelay:", stat.MaxDelay, " MinDelay:", stat.MinDelay, " AvgDelay:", stat.AvgDelay, " Revc:", stat.RevcPk, " LossPK:", stat.LossPk)
	} else {
		stat.AvgDelay = 0.00
		stat.MinDelay = 0.00
		stat.MaxDelay = 0.00
		stat.SendPk = 0
		stat.RevcPk = 0
		stat.LossPk = 100
		glog.Debug(ctx, "[func:IcmpPing] Finish Addr:", addr, " Unable to resolve destination host")
	}

	//添加到数据库
	AddPingLog(ctx, stat, t, serverid)
	wg.Done()
}

func AddPingLog(ctx context.Context, pingres model.PingSt, addr string, serverid int) {
	err := service.Ping().Create(ctx, pingres, serverid)
	if err != nil {
		glog.Debug(ctx, err.Error())
		return
	}
	err = service.Node().UpdateNodeMs(ctx, model.NodeUpdateInput{
		ServerId: serverid,
		NodeMs:   pingres.AvgDelay,
	})
	if err != nil {
		glog.Debug(ctx, err.Error())
		return
	}
	return
}

func CheckNodeStatus(ctx context.Context, nextTime time.Time) {
	glog.Info(ctx, nextTime)
	nodeList, err := service.Node().GetNode(ctx)
	if err != nil {
		glog.Error(ctx, err.Error())
		return
	}
	for _, target := range nodeList {
		//获取不通的IP进程url请求
		status, err := service.Ping().GetStatus(ctx, target.Id)
		if err != nil {
			glog.Error(ctx, err.Error())
			return
		}
		if status {

			node, err := service.Node().GetNodeUrlCount(ctx, target.Id)
			if err != nil {
				glog.Error(ctx, err.Error())
				return
			}

			if target.Url != "" {
				urlcount := node.UrlCount
				urlret := ""
				ipstatus := 0
				glog.Error(ctx, "start url req .....")
				s := fmt.Sprintf("url_log ---- 开始请求%s\n", target.Url)
				websocket.SendToAll(&websocket.WResponse{
					Event: "sys_log",
					Data: g.Map{
						"msg": s,
					},
				})
				client := g.Client()
				client.SetTimeout(30 * time.Second)
				r, err := client.Get(ctx, target.Url)
				if err != nil {
					s := fmt.Sprintf("url_log ---- 请求错误%s\n", err.Error())
					websocket.SendToAll(&websocket.WResponse{
						Event: "sys_log",
						Data: g.Map{
							"msg": s,
						},
					})
					glog.Error(ctx, err.Error())
					return
				}
				defer r.Close()
				glog.Info(ctx, "req :", target.Url, "status :", r.Status)
				//如果成功是放回IP，如果不成功是返回空
				if r.StatusCode == 200 {
					urlret = r.ReadAllString()
					s := fmt.Sprintf("url_log ---- url结果和状态%s --- %d\n", urlret, r.StatusCode)
					websocket.SendToAll(&websocket.WResponse{
						Event: "sys_log",
						Data: g.Map{
							"msg": s,
						},
					})
				}
				urlcount++
				s = fmt.Sprintf("url_log --- url执行次数%d\n", urlcount)
				websocket.SendToAll(&websocket.WResponse{
					Event: "sys_log",
					Data: g.Map{
						"msg": s,
					},
				})
				err = service.Node().UPdateNodeUrlCount(ctx, target.Id, urlcount, urlret, ipstatus)
				if err != nil {
					glog.Error(ctx, err.Error())
					return
				}
			} else {
				if target.IsNotice == 1 {

					s := fmt.Sprintf("telegram_log --- %s 节点需要更新换IP,立即执行\n", target.Host)
					websocket.SendToAll(&websocket.WResponse{
						Event: "sys_log",
						Data: g.Map{
							"msg": s,
						},
					})
					err := telegram.SetTelegramMess(ctx, s)
					if err != nil {
						glog.Error(ctx, err.Error())
						s := fmt.Sprintf("telegram_log --- 电报发送: %s", err.Error())
						websocket.SendToAll(&websocket.WResponse{
							Event: "sys_log",
							Data: g.Map{
								"msg": s,
							},
						})
						return
					}
				}
			}

			SSHTaskCommand(ctx, target.Host, target.Port, target.Id)

		}
	}

}

func SSHTaskCommand(ctx context.Context, host string, port int, serverid int) {

	nodeconfig, err := service.NodeConfig().Get(ctx, serverid)
	if err != nil {
		glog.Error(ctx, err.Error())
		return
	}

	if nodeconfig.Sshcommand == "" && nodeconfig.Sshpassword == "" && nodeconfig.Sshusername == "" {
		glog.Info(ctx, "服务器命令账号密码为空")
		websocket.SendToAll(&websocket.WResponse{
			Event: "sys_log",
			Data: g.Map{
				"msg": "ssh_log --- 服务器命令账号密码为空",
			},
		})
		return
	}

	s2 := fmt.Sprintf("%s:%d\n", host, port)
	// 建立SSH客户端连接
	client, err := ssh.Dial("tcp", s2, &ssh.ClientConfig{
		User:            nodeconfig.Sshusername,
		Auth:            []ssh.AuthMethod{ssh.Password(nodeconfig.Sshpassword)},
		HostKeyCallback: ssh.InsecureIgnoreHostKey(),
	})
	if err != nil {
		//log.Fatalf("SSH dial error: %s", err.Error())
		s := fmt.Sprintf("ssh_log --- SSH dial error: %s", err.Error())
		websocket.SendToAll(&websocket.WResponse{
			Event: "sys_log",
			Data: g.Map{
				"msg": s,
			},
		})
		glog.Error(ctx, "SSH dial error:", err.Error())
		return
	}
	// 建立新会话
	session, err := client.NewSession()
	defer session.Close()
	if err != nil {
		s := fmt.Sprintf("ssh_log --- new session error: %s", err.Error())
		websocket.SendToAll(&websocket.WResponse{
			Event: "sys_log",
			Data: g.Map{
				"msg": s,
			},
		})
		glog.Error(ctx, "new session error:", err.Error())
		return
	}

	s := fmt.Sprintf("ssh_log --- 执行命令 %s", nodeconfig.Sshcommand)
	websocket.SendToAll(&websocket.WResponse{
		Event: "sys_log",
		Data: g.Map{
			"msg": s,
		},
	})
	result, err := session.Output(nodeconfig.Sshcommand)
	if err != nil {
		glog.Error(ctx, err.Error())
		return
	}
	s = fmt.Sprintf("ssh_log ---- 执行命令结果 %s", string(result))
	websocket.SendToAll(&websocket.WResponse{
		Event: "sys_log",
		Data: g.Map{
			"msg": s,
		},
	})
	glog.Info(ctx, result)
}

func GetCronNextTime(cronStr string, t time.Time) (nextTime time.Time, err error) {
	p := cron.NewParser(cron.Second | cron.Minute | cron.Hour | cron.Dom | cron.Month | cron.Dow)
	s, err := p.Parse(cronStr)
	if err != nil {
		return
	}
	nextTime = s.Next(t)
	return
}

func ClearLog(ctx context.Context) {
	service.Ping().ClearLog(ctx)
	service.Node().UpdateAllNodeMs(ctx)
}
